/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "datasetchain.h"
#include "datasetchain.moc"
//
// Both Queues seem to be equally good.
//
//#include "simplesafequeue.h"
#include <queue>

#include <QtCore/QDebug>
#include <QtCore/QThread>
#include <QtCore/QCoreApplication>
#include <QtCore/QTime>
#include <QtCore/QTimer>
#include <QtCore/QReadLocker>
#include <QtCore/QWriteLocker>
#include <QtCore/QEvent>

namespace NSCore {

	const QEvent::Type MyEvent = (  QEvent::Type )1003;
	class DataSetEvent : public QEvent
	{
		public:
			DataSetEvent( AcqisitionDataSet* d ) : QEvent( MyEvent ), data( d ) {
				//qDebug() << "DataSetEvent::DataSetEvent(" << d << ")";
			}
			AcqisitionDataSet* data;
	};

	class DataSetProducer::DataSetProducerPrivate {
		public:
			QList< Pointer_template<DataSetChainMember> > consumers;
			DataSetProducerPrivate()
			{
				time = QTime::currentTime();
				time.start();
				counter = 0;
			}
			~DataSetProducerPrivate() {
			}

			int counter;
			QTime time;
	};

	DataSetProducer::DataSetProducer( QObject* p ) : QTreeItem( p ), _d( new DataSetProducerPrivate() ) {
		qDebug() << "DataSetProducer::DataSetProducer() living in thread" << QThread::currentThread();
	}
	DataSetProducer::~DataSetProducer() {
		//qDebug() << "DataSetProducer::~DataSetProducer()";
		while ( ! _d->consumers.isEmpty() ) {
			Pointer_template<DataSetChainMember> pointer = _d->consumers.takeFirst();
			if ( * pointer != 0 )
				pointer->unregisterProducer( this );
		}
		delete _d;
	}

	void DataSetProducer::registerConsumer( DataSetChainMember* consumer ) {
		//qDebug() << "DataSetProducer::registerConsumer(" << consumer << ")";
		if ( consumer == 0 )
			qWarning() << "Tried to register Consumer with address 0!";
		else
			_d->consumers.append( Pointer_template<DataSetChainMember>( consumer ) );
	}
	void DataSetProducer::unregisterConsumer( DataSetChainMember* consumer ) {
		foreach( Pointer_template<DataSetChainMember> pointer, _d->consumers ) {
			if ( pointer.object() == consumer )
				_d->consumers.removeAll( pointer );
		}
	}

	void DataSetProducer::enqueue( AcqisitionDataSet* data ) {
		//qDebug() << "DataSetProducer::enqueue(" << data << ")";
		if ( _d->time.elapsed() >= 1000 ) {
			emit countsPerSecond( _d->counter/( _d->time.elapsed()/1000.0 ) );
			_d->time.restart();
			_d->counter = 0;
		}
		if ( !_d->consumers.isEmpty() ) {
			foreach( Pointer_template<DataSetChainMember> pointer, _d->consumers ) {
				DataSetEvent* ev = new DataSetEvent( new NSCore::AcqisitionDataSet( *data ) );
				//qDebug() << "Will post event to" << *( _d->consumer );
				QCoreApplication::postEvent( pointer.object(), ev );
			}
		}
		delete data;
		_d->counter++;
	}


	DataSetChainMember::DataSetChainMember( QObject* p )
		: DataSetProducer( p )
	{
		qDebug() << "DataSetChainMember::DataSetChainMember()";
	}
	DataSetChainMember::~DataSetChainMember() {
		//qDebug() << "DataSetChainMember::~DataSetChainMember()";
		while ( !_producers.empty() ) {
			if ( _producers.front() != 0 )
				_producers.front()->unregisterConsumer( this );
			_producers.pop_front();
		}
	}
	void DataSetChainMember::registerAtProducer( DataSetProducer* producer ) {
		_producers.push_back( producer );
		producer->registerConsumer( this );
	}
	void DataSetChainMember::unregisterProducer( DataSetProducer* producer ) {
		//qDebug() << "DataSetChainMember::unregisterProducer(" << producer << ") this =" << this;
		_producers.remove( producer );
	}

	bool DataSetChainMember::event( QEvent* ev ) {
		if ( ev->type() == MyEvent ) {
			//qDebug() << "DataSetChainMember::event(" << ev << ") Got my own event-type!\n this=" << this;
			DataSetEvent* e = dynamic_cast<DataSetEvent*>( ev );
			if ( e ) {
				if ( dataArrived( e->data ) )
					enqueue( e->data );
			}
			return true;
		}
		return QObject::event( ev );
	}

};

