/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include <math.h>

#include "mathexpression.h"
#include "mathexpression.moc"

using namespace NSCore;

MathExpression::MathExpression( NSCore::QTreeItem* parent )
	: NSCore::QTreeItem( parent )
{
	setObjectName( "MathExpression" );
}

QString MathExpression::guiString() const { return objectName(); }
QStringList MathExpression::guiActions() const { return NSCore::QTreeItem::guiActions() << "Remove me"; }

void MathExpression::guiExecAction( QString name, NSCore::GuiServer_Interface* gui ) {
	//qDebug() << "MathExpression::guiExecAction(" << name << "," << gui << ")";
	if ( name == "Remove me" ) {
		deleteMe( this, gui );
		return;
	}
	if ( name == "Add Exp." ) {
		addChild( this, gui );
		return;
	}
	NSCore::QTreeItem::guiExecAction( name, gui );
}

void MathExpression::deleteMe( MathExpression* exp, NSCore::GuiServer_Interface* gui ) {
	//qDebug() << "MathExpression::deleteMe(" << exp << "," << gui << ")";
	MathExpression* p = dynamic_cast<MathExpression*>( parent() );
	if ( p )
		p->deleteMe( exp, gui );
}
void MathExpression::addChild( MathExpression* pexp, NSCore::GuiServer_Interface* gui ) {
	//qDebug() << "MathExpression::addChild(" << pexp << "," << gui << ")";
	MathExpression* p = dynamic_cast<MathExpression*>( parent() );
	if ( p )
		p->addChild( pexp, gui );
}


MathExpressionFromXMLCreator::MathExpressionFromXMLCreator() {
	//qDebug() << "MathExpressionFromXMLCreator::MathExpressionFromXMLCreator()";
	the()->_creators.push_back( this );
}
MathExpressionFromXMLCreator* MathExpressionFromXMLCreator::the() {
	static MathExpressionFromXMLCreator* me = 0;
	if ( !me ) {
		//qDebug() << "MathExpressionFromXMLCreator::the() me is null, will create myself...";
		me = new MathExpressionFromXMLCreator( true );
		me->_creators.push_back( me );
	}
	return me;
}
MathExpression* MathExpressionFromXMLCreator::fromXML( QString xml, QTreeItem* pexp ) {
	QDomDocument doc( "bla" );
	doc.setContent( xml );
	return fromXML( doc, pexp );
}
MathExpression* MathExpressionFromXMLCreator::fromXML( QDomNode node, QTreeItem* pexp ) {
	qDebug() << "MathExpressionFromXMLCreator::fromXML(" << &node << "," << pexp << ")";
	if ( node.nodeName() == "#document" )
		node = node.toDocument().documentElement();

	MathExpression* ret = 0;
	QList<MathExpressionFromXMLCreator*>::iterator it = _creators.begin();
	while ( ret == 0 && it != _creators.end() ) {
		qDebug() << " Trying" << *it << "for next element.";
		ret = ( *it )->singleElementFromXML( node, pexp );
		it++;
	}
	QDomNode child = node.firstChild();
	while ( !child.isNull() ) {
		fromXML( child, ret );
		child = child.nextSibling();
	}
	return ret;
}
QStringList MathExpressionFromXMLCreator::availableElements() const {
	QStringList ret;
	foreach( MathExpressionFromXMLCreator* c, _creators )
		ret << c->elementsICanCreate();
	return ret;
}
QString MathExpressionFromXMLCreator::helpText( QString n ) const {
	QString ret;
	QList<MathExpressionFromXMLCreator*>::const_iterator it = _creators.begin();
	while ( ret.isNull() && it != _creators.end() ) {
		ret = ( *it )->myHelpText( n );
		it++;
	}
	return ret;
}

MathExpression* MathExpressionFromXMLCreator::singleElementFromXML( QDomNode node, QTreeItem* pexp ) {
	qDebug() << "MathExpressionFromXMLCreator::singleElementFromXML(" << &node << "," << pexp << ")";

	MathExpression* returnvalue = 0;
	if ( node.nodeName() == "result" ) {
		QString name;
		if ( ! node.attributes().namedItem( "name" ).isNull() )
			name = node.attributes().namedItem( "name" ).nodeValue();
		int channel = 0;
		if ( ! node.attributes().namedItem( "channel" ).isNull() )
			channel = node.attributes().namedItem( "channel" ).nodeValue().toInt();
		int generation = -1000;
		if ( ! node.attributes().namedItem( "generation" ).isNull() )
			generation = node.attributes().namedItem( "generation" ).nodeValue().toInt();
		returnvalue = new NSCore::MathExpression_Result( name, channel, generation, pexp );
	}
	if ( node.nodeName() == "sampleparameter" ) {
		QString name;
		if ( ! node.attributes().namedItem( "name" ).isNull() )
			name = node.attributes().namedItem( "name" ).nodeValue();
		int channel = -1;
		if ( ! node.attributes().namedItem( "channel" ).isNull() )
			channel = node.attributes().namedItem( "channel" ).nodeValue().toInt();
		returnvalue = new NSCore::MathExpression_Parameter( name, channel, pexp );
	}
	if ( node.nodeName() == "constant" ) {
		double value = NAN;
		if ( ! node.attributes().namedItem( "value" ).isNull() )
			value = node.attributes().namedItem( "value" ).nodeValue().toDouble();
		returnvalue = new NSCore::MathExpression_Constant( value, pexp );
	}
	if ( node.nodeName() == "smb" ) {
		QString selection, calculus;
		int generation = -1000;
		int generation_calc = -2000;
		if ( ! node.attributes().namedItem( "selection" ).isNull() )
			selection = node.attributes().namedItem( "selection" ).nodeValue();
		if ( ! node.attributes().namedItem( "calculus" ).isNull() )
			calculus = node.attributes().namedItem( "calculus" ).nodeValue();
		if ( ! node.attributes().namedItem( "generation" ).isNull() )
			generation = node.attributes().namedItem( "generation" ).nodeValue().toInt();
		if ( ! node.attributes().namedItem( "generation_select" ).isNull() )
			generation = node.attributes().namedItem( "generation_select" ).nodeValue().toInt();
		if ( ! node.attributes().namedItem( "generation_calc" ).isNull() )
			generation_calc = node.attributes().namedItem( "generation_calc" ).nodeValue().toInt();
		returnvalue = new NSCore::MathExpression_SmB( selection, calculus, generation, pexp, generation_calc );
	}
	if ( node.nodeName() == "add" ) {
		returnvalue = new NSCore::MathExpression_Add( pexp );
	}
	if ( node.nodeName() == "mul" ) {
		returnvalue = new NSCore::MathExpression_Mul( pexp );
	}
	if ( node.nodeName() == "min" ) {
		returnvalue = new NSCore::MathExpression_Min( pexp );
	}
	if ( node.nodeName() == "max" ) {
		returnvalue = new NSCore::MathExpression_Max( pexp );
	}
	if ( node.nodeName() == "neg" ) {
		returnvalue = new NSCore::MathExpression_Neg( pexp );
	}
	if ( node.nodeName() == "precision" ) {
		double precision = 1E-3;
		if ( ! node.attributes().namedItem( "precision" ).isNull() )
			precision = node.attributes().namedItem( "precision" ).nodeValue().toDouble();
		returnvalue = new NSCore::MathExpression_Precision( precision, pexp );
	}
	if ( node.nodeName() == "gtz" ) {
		returnvalue = new NSCore::MathExpression_GtZ( pexp );
	}
	if ( node.nodeName() == "ltz" ) {
		returnvalue = new NSCore::MathExpression_LtZ( pexp );
	}
	if ( node.nodeName() == "window" ) {
		double min = 0, max = 0;
		if ( ! node.attributes().namedItem( "min" ).isNull() )
			min = node.attributes().namedItem( "min" ).nodeValue().toDouble();
		if ( ! node.attributes().namedItem( "max" ).isNull() )
			max = node.attributes().namedItem( "max" ).nodeValue().toDouble();
		returnvalue = new MathExpression_Window( min, max, pexp );
	}
	return returnvalue;
}
QStringList MathExpressionFromXMLCreator::elementsICanCreate() const {
	return QStringList() << "result" << "sampleparameter" << "constant" << "smb" << "precision" << "add" << "mul" << "neg" << "min" << "max" << "gtz" << "ltz" << "window";
};
QString MathExpressionFromXMLCreator::myHelpText( QString n ) const {
	qDebug() << "MathExpressionFromXMLCreator::myHelpText(" << n.toStdString().c_str() << ") Not implemented yet!";
	if ( n == "result" ) return "Get a results value from the data.";
	if ( n == "sampleparameter" ) return "Get a parameter from the data.";
	if ( n == "constant" ) return "A constant value.";
	if ( n == "smb" ) return "<b>Smaller-Minus-Bigger</b>: decide which channels value to substract by another result.";
	if ( n == "precision" ) return "Reduce the precision of a value.";
	if ( n == "add" ) return "Addition of values.";
	if ( n == "mul" ) return "Multiply values.";
	if ( n == "neg" ) return "Negate the value.";
	if ( n == "min" ) return "Return the minimum of a number of values.";
	if ( n == "max" ) return "Return the maximum of a number of values.";
	if ( n == "gtz" ) return "Pass the value if its greater than zero, otherwise pass NaN.";
	if ( n == "ltz" ) return "Pass the value if its less than zero, otherwise pass NaN.";
	if ( n == "window" ) return "Pass the value (or the second value if present) if its inside the given window, otherwise pass NaN (or the third value if present).";
	//return "Help for this element is not yet implemented!";
	return "";
}


MathExpression_Result::MathExpression_Result( QString name, int channel, int generation, NSCore::QTreeItem* parent )
	: MathExpression( parent )
	, _name( new NSCore::NamedParameterTreeItem( "name", name, this ) )
	, _channel( new NSCore::NamedParameterTreeItem( "channel", channel, this ) )
	, _generation( new NSCore::NamedParameterTreeItem( "generation", generation, this ) )
{
	//maximumChilds( 3 );
	_channel->setMin( 0 );
	_channel->setMax( 10 );
	setObjectName( "Result" );
}

double MathExpression_Result::value( NSCore::AcqisitionDataSet& data ) {
	//qDebug() << "MathExpression_Result::value( data )";
	double ret=NAN;
	for ( int i=0; i<data.channels(); ++i ) {
		if ( channel() == i ) {
			QVariant* tmp = data.channel( i )->result( name(), generation() );
			//qDebug() << i << "\t tmp =" << tmp;
			if ( tmp )
				ret = tmp->toDouble();
		}
	}
	//qDebug() << " returning" << ret;
	return ret;
}

QString MathExpression_Result::toXML() const {
	return QString( "<result name=\"%1\" channel=\"%2\" generation=\"%3\" />" ).arg( _name->toString() ).arg( _channel->toString() ).arg( _generation->toString() );
}

MathExpression_Parameter::MathExpression_Parameter( QString name, int channel, NSCore::QTreeItem* parent )
	: MathExpression( parent )
	, _name( new NSCore::NamedParameterTreeItem( "name", name, this ) )
	, _channel( new NSCore::NamedParameterTreeItem( "channel", channel, this ) )
{
	//maximumChilds( 2 );
	_channel->setMin( -1 );
	_channel->setMax( 10 );
	setObjectName( "Parameter" );
}

double MathExpression_Parameter::value( NSCore::AcqisitionDataSet& data ) {
	double ret=NAN;
	int ch = channel();
	if ( ch < 0 ) {
		QVariant* tmp = data.parameter( name() );
		if ( tmp )
			ret = tmp->toDouble();
	} else
		if ( ch < data.channels() ) {
			QVariant* tmp = data.channel( ch )->parameter( name() );
			if ( tmp )
				ret = tmp->toDouble();
		}
	if ( isnan( ret ) )
		qWarning() << " Parameter" << name() << " on channel" << ch << " returns NAN";

	//qDebug() << " Parameter" << name() << "on channel" << ch << "returns" << ret;

	return ret;
}

QString MathExpression_Parameter::toXML() const {
	return QString( "<sampleparameter name=\"%1\" channel=\"%2\" />" ).arg( _name->toString() ).arg( _channel->toString() );
}

MathExpression_Constant::MathExpression_Constant( double value, NSCore::QTreeItem* parent )
	: MathExpression( parent )
	, _value( new NSCore::NamedParameterTreeItem( "value", value, this ) )
{
	//maximumChilds( 1 );
	_value->setMin( -INFINITY );
	_value->setMax( INFINITY );
	setObjectName( "Constant" );
}

double MathExpression_Constant::value( NSCore::AcqisitionDataSet& data ) {
	return _value->toDouble();
}

QString MathExpression_Constant::toXML() const {
	return QString( "<constant value=\"%1\" />" ).arg( _value->toString() );
}

MathExpression_Add::MathExpression_Add( NSCore::QTreeItem* parent ) : MathExpression( parent ) {
	setObjectName( "Add" );
}

double MathExpression_Add::value( NSCore::AcqisitionDataSet& data ) {
	double ret = 0;
	foreach( QObject* item, childobjects() ) {
		MathExpression* exp = dynamic_cast<MathExpression*>( item );
		if ( exp != 0 )
			ret += exp->value( data );
	}
	return ret;
}

QString MathExpression_Add::toXML() const {
	QString ret = "<add>";
	foreach( QObject* item, childobjects() ) {
		MathExpression* exp = dynamic_cast<MathExpression*>( item );
		if ( exp )
			ret += exp->toXML();
	}
	ret += "</add>";
	return ret;
}

MathExpression_Mul::MathExpression_Mul( NSCore::QTreeItem* parent ) : MathExpression( parent ) {
	setObjectName( "Multiplication" );
}

double MathExpression_Mul::value( NSCore::AcqisitionDataSet& data ) {
	double ret = 1;
	foreach( QObject* item, childobjects() ) {
		MathExpression* exp = dynamic_cast<MathExpression*>( item );
		if ( exp != 0 )
			ret *= exp->value( data );
	}
	return ret;
}

QString MathExpression_Mul::toXML() const {
	QString ret = "<mul>";
	foreach( QObject* item, childobjects() ) {
		MathExpression* exp = dynamic_cast<MathExpression*>( item );
		if ( exp )
			ret += exp->toXML();
	}
	ret += "</mul>";
	return ret;
}

MathExpression_Min::MathExpression_Min( NSCore::QTreeItem* parent ) : MathExpression( parent ) {
	setObjectName( "Minimum" );
}

double MathExpression_Min::value( NSCore::AcqisitionDataSet& data ) {
	double ret = INFINITY;
	foreach( QObject* item, childobjects() ) {
		MathExpression* exp = dynamic_cast<MathExpression*>( item );
		if ( exp != 0 ) {
			/*double val = exp->value( data );
			if ( val < ret )
				ret = val;*/
			ret = qMin( ret, exp->value( data ) );
		}
	}
	return ret;
}

QString MathExpression_Min::toXML() const {
	QString ret = "<min>";
	foreach( QObject* item, childobjects() ) {
		MathExpression* exp = dynamic_cast<MathExpression*>( item );
		if ( exp )
			ret += exp->toXML();
	}
	ret += "</min>";
	return ret;
}
MathExpression_Max::MathExpression_Max( NSCore::QTreeItem* parent ) : MathExpression( parent ) {
	setObjectName( "Maximum" );
}

double MathExpression_Max::value( NSCore::AcqisitionDataSet& data ) {
	double ret = -INFINITY;
	foreach( QObject* item, childobjects() ) {
		MathExpression* exp = dynamic_cast<MathExpression*>( item );
		if ( exp != 0 ) {
			/*double val = exp->value( data );
			if ( val < ret )
				ret = val;*/
			ret = qMax( ret, exp->value( data ) );
		}
	}
	return ret;
}

QString MathExpression_Max::toXML() const {
	QString ret = "<max>";
	foreach( QObject* item, childobjects() ) {
		MathExpression* exp = dynamic_cast<MathExpression*>( item );
		if ( exp )
			ret += exp->toXML();
	}
	ret += "</max>";
	return ret;
}


MathExpression_Neg::MathExpression_Neg( NSCore::QTreeItem* parent )
	: MathExpression( parent )
{
	//maximumChilds( 1 );
	setObjectName( "Negation" );
}

double MathExpression_Neg::value( NSCore::AcqisitionDataSet& data ) {
	double ret = NAN;
	foreach( QObject* item, childobjects() ) {
		MathExpression* exp = dynamic_cast<MathExpression*>( item );
		if ( exp != 0 )
			ret = - exp->value( data );
	}
	return ret;
}

QString MathExpression_Neg::toXML() const {
	QString ret = "<neg>";
	foreach( QObject* item, childobjects() ) {
		MathExpression* exp = dynamic_cast<MathExpression*>( item );
		if ( exp )
			ret += exp->toXML();
	}
	return ret+"</neg>";
}

MathExpression_Precision::MathExpression_Precision( double precision, NSCore::QTreeItem* p )
	: MathExpression( p )
	, _precision( new NSCore::NamedParameterTreeItem( "precision", precision, this ) )
{
	//maximumChilds( 2 );
	setObjectName( "Precision" );
}

double MathExpression_Precision::value( NSCore::AcqisitionDataSet& data ) {
	double ret = NAN;
	foreach( QObject* item, childobjects() ) {
		MathExpression* exp = dynamic_cast<MathExpression*>( item );
		if ( exp != 0 )
			ret = exp->value( data );
	}
	ret = ret - fmod( ret, _precision->toDouble() );
	return ret;
}

QString MathExpression_Precision::toXML() const {
	QString ret = QString( "<precision precision=\"%1\">" ).arg( _precision->toString() );
	foreach( QObject* item, childobjects() ) {
		MathExpression* exp = dynamic_cast<MathExpression*>( item );
		if ( exp )
			ret += exp->toXML();
	}
	return ret+"</precision>";
}

MathExpression_SmB::MathExpression_SmB( QString selection, QString calculus, int generation, NSCore::QTreeItem* p, int generation_calc )
	: MathExpression( p )
	, _selection( new NSCore::NamedParameterTreeItem( "selection", selection, this ) )
	, _calculus( new NSCore::NamedParameterTreeItem( "calculus", calculus, this ) )
	, _generation_select( new NSCore::NamedParameterTreeItem( "selection generation", generation, this ) )
	, _generation_calc( new NSCore::NamedParameterTreeItem( "calculus generation", generation_calc, this ) )
{
	//maximumChilds( 3 );
	setObjectName( "SmB" );
}

double MathExpression_SmB::value( NSCore::AcqisitionDataSet& data ) {
	double ret = NAN;
	double _sel1=NAN, _sel2=NAN, _cal1=NAN, _cal2=NAN;
	int gen_select = _generation_select->toInt();
	int gen_calc = _generation_calc->toInt();
	if ( gen_calc == -2000 )
		gen_calc = gen_select;
	QString sel = _selection->toString();
	QString cal = _calculus->toString();
	if ( data.channel( 0 ) != 0 ) {
		if ( data.channel( 0 )->result( sel, gen_select ) != 0 )
			_sel1 = data.channel( 0 )->result( sel, gen_calc )->toDouble();
		if ( data.channel( 0 )->result( cal, gen_select ) != 0 )
			_cal1 = data.channel( 0 )->result( cal, gen_calc )->toDouble();
	}
	if ( data.channel( 1 ) != 0 ) {
		if ( data.channel( 1 )->result( sel, gen_select ) != 0 )
			_sel2 = data.channel( 1 )->result( sel, gen_calc )->toDouble();
		if ( data.channel( 1 )->result( cal, gen_select ) != 0 )
			_cal2 = data.channel( 1 )->result( cal, gen_calc )->toDouble();
	}
	if ( _sel1 < _sel2 )
		ret = _cal1 - _cal2;
	else
		if ( _sel2 <= _sel1 )
			ret = _cal2 - _cal1;

	return ret;
}

QString MathExpression_SmB::toXML() const {
	return QString( "<smb selection=\"%1\" calculus=\"%2\" generation_select=\"%3\" generation_calc=\"%4\" />" ).arg( _selection->toString() ).arg( _calculus->toString() ).arg( _generation_select->toString() ).arg( _generation_calc->toString() );
}

MathExpression_GtZ::MathExpression_GtZ( NSCore::QTreeItem* p )
	: MathExpression( p ) {}

double MathExpression_GtZ::value( NSCore::AcqisitionDataSet& data ) {
	double ret = NAN;
	if ( childs() > 1 ) {
		MathExpression* tmp = dynamic_cast<MathExpression*>( child( 0 ) );
		if ( tmp && tmp->value( data ) > 0 ) {
			tmp = dynamic_cast<MathExpression*>( child( 1 ) );
			if ( tmp )
				ret = tmp->value( data );
		} else {
			tmp = dynamic_cast<MathExpression*>( child( 2 ) );
			if ( tmp )
				ret = tmp->value( data );
		}
	}
	return ret;
}
QString MathExpression_GtZ::toXML() const {
	if ( childs() > 0 ) {
		QString ret = "<gtz>";
		foreach( QObject* item, childobjects() ) {
			MathExpression* exp = dynamic_cast<MathExpression*>( item );
			if ( exp )
				ret += exp->toXML();
		}
		return ret + "</gtz>";
	} else
		return "<gtz />";
}

MathExpression_LtZ::MathExpression_LtZ( NSCore::QTreeItem* p )
	: MathExpression( p ) {}

double MathExpression_LtZ::value( NSCore::AcqisitionDataSet& data ) {
	double ret = NAN;
	if ( childs() > 1 ) {
		MathExpression* tmp = dynamic_cast<MathExpression*>( child( 0 ) );
		if ( tmp && tmp->value( data ) < 0 ) {
			tmp = dynamic_cast<MathExpression*>( child( 1 ) );
			if ( tmp )
				ret = tmp->value( data );
		} else {
			tmp = dynamic_cast<MathExpression*>( child( 2 ) );
			if ( tmp )
				ret = tmp->value( data );
		}
	}
	return ret;
}
QString MathExpression_LtZ::toXML() const {
	if ( childs() > 0 ) {
		QString ret = "<ltz>";
		foreach( QObject* item, childobjects() ) {
			MathExpression* exp = dynamic_cast<MathExpression*>( item );
			if ( exp )
				ret += exp->toXML();
		}
		return ret + "</ltz>";
	} else
		return "<ltz />";
}
MathExpression_Window::MathExpression_Window( double min, double max, NSCore::QTreeItem* p )
	: MathExpression( p )
	, _min( new NamedParameterTreeItem( "minimum", min, this ) )
	, _max( new NamedParameterTreeItem( "maximum", max, this ) )
{
}

double MathExpression_Window::value( NSCore::AcqisitionDataSet& data ) {
	double ret = NAN;
	double min = _min->toDouble();
	double max = _max->toDouble();
	QList<MathExpression*> childs = childobjects<MathExpression>();
	if ( childs.size() > 0 ) {
		double tmp = childs[ 0 ]->value( data );
		if ( childs.size() == 1 ) {
			if ( min < tmp && tmp < max )
				ret = tmp;
		} else {
			if ( min < tmp && tmp < max )
				ret = childs[ 1 ]->value( data );
			else
				if ( childs.size() > 2 )
					ret = childs[ 2 ]->value( data );
		}
	}
	return ret;
}
QString MathExpression_Window::toXML() const {
	QString ret = QString( "<window min=\"%1\" max=\"%2\" " ).arg( _min->toDouble() ).arg( _max->toDouble() );
	QList<MathExpression*> list = childobjects<MathExpression>();
	if ( list.size() > 0 ) {
		ret += ">";
		for ( int i=0; i<list.size(); ++i )
			ret += list[ i ]->toXML();
		ret += "</window>";
	} else
		ret += "/>";
	return ret;
}


