/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef NAMEDQVARIANTTREEITEM_H
#define NAMEDQVARIANTTREEITEM_H

#include <QtCore/QVariant>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QDebug>
#include "pointer_template.h"

#include "abstracttreeitem.h"

namespace NSCore {

	/**
	 * @brief Stores QVariant in a AbstractTreeItem structure
	 *
	 * This class complements NamedParameterTreeItem. It also stores values but
	 * has no signals or slots for manipulating and therefor no dependency on
	 * QObject which makes it great for results that aren't changed from various
	 * threads and guis.
	 */
	class NamedQVariantTreeItem : public AbstractTreeItem, public QVariant
	{
		public:
			/// Constructs a new named QVariant as child of the parent AbstractTreeItem
			NamedQVariantTreeItem( QString, QVariant, AbstractTreeItem* =0 );

			/// Constructs a recursiv copy of the given NamedQVariantTreeItem as a child of the parent NamedQVariantTreeItem
			NamedQVariantTreeItem( NamedQVariantTreeItem&, AbstractTreeItem* =0 );
			NamedQVariantTreeItem( NamedQVariantTreeItem*, AbstractTreeItem* );

			/// Deconstructor
			~NamedQVariantTreeItem();

			/// Returns a child QVariant by path
			NamedQVariantTreeItem* getQVariant( QString );

			/// Adds a QVariant as a named child
			void addQVariant( QString, QVariant );

			/**
			 * @brief Set the value from a QVariant
			 *
			 * QVariant is missing one very important function: setting the
			 * value from another QVariant!
			 * But only if the types are the same...
			 */
			void setValue( QVariant );

		private:
			QString debug() const {
				QString tmp = QString( "(%1,%2)" ).arg( toString() ).arg( QVariant::typeToName( type() ) );
				if ( tmp == "(,)" )
					tmp="";
				return QString( "%1%2" ).arg( name() ).arg( tmp );
			}
	};

};

#endif // NAMEDQVARIANTTREEITEM_H

