/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef DISKIO_GUI_H
#define DISKIO_GUI_H

#include <QtGui/QFrame>

class QSpinBox;
class QToolBar;
class QAction;
class QLabel;
class QLineEdit;

namespace NSDiskIO {

	class AbstractReader;
	class AbstractWriter;

	class DiskReaderGui : public ::QFrame {
		Q_OBJECT
		public:
			DiskReaderGui( AbstractReader*, QWidget* );

			QToolBar* toolbar() { return _toolbar; }

		public slots:
			void startedReading();
			void stoppedReading();

		private slots:
			void readn();
			void openfile();

			void stopReading();

		signals:
			void readn( int );

		private:
			void contextMenuEvent( QContextMenuEvent* );
			AbstractReader* _reader;
			QSpinBox* _spinbox;
			QToolBar* _toolbar;
			QAction *_fileopen, *_readAll, *_readN, *_reset, *_stopreading;
	};

	class DiskWriterGui : public ::QFrame {
		Q_OBJECT
		public:
			DiskWriterGui( AbstractWriter*, QWidget* );

		public slots:
			void wroteDataSets( int );
			void setNewFilename();
			void closeFile();

		signals:
			void numberOfDataSetsReached();

		private:
			AbstractWriter* _writer;
			QLabel *_lblcount, *_lblfilename;
			QLineEdit *_inputprefix;
			QSpinBox *_spinboxnumberofdatasets;

			QString _filename;
	};
};

#endif // DISKIO_GUI_H

