/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef FILTERS_H
#define FILTERS_H

#include <plugins_interface.h>

class PeakFilter : public NSEvaluation::Filter
{
	public:
		PeakFilter( NSCore::QTreeItem* );

		bool filter( NSCore::AcqisitionDataSet& );

		QString type() const { return "PeakFilter"; }
};

class TwoChannelDisc : public NSEvaluation::Filter
{
	public:
		TwoChannelDisc( NSCore::QTreeItem* );

		bool filter( NSCore::AcqisitionDataSet& );

		QString type() const { return "TwoChannelDisc"; }

	private:
		NSEvaluation::Analyzer* _analyzer;
};

class TwoChannelXORDisc : public NSEvaluation::Filter
{
	public:
		TwoChannelXORDisc( NSCore::QTreeItem* );

		bool filter( NSCore::AcqisitionDataSet& );

		QString type() const { return "TwoChannelXORDisc"; }

		QString help() const { return "<qt>This rebuilds the windows of traditional cf-discs without the need to fix start- and stop-tube.</qt>"; }

	private:
		NSEvaluation::Analyzer* _analyzer;
};

class SimpleDisc : public NSEvaluation::Filter
{
	public:
		SimpleDisc( NSCore::QTreeItem* );

		bool filter( NSCore::AcqisitionDataSet& );

		QString type() const { return "SimpleDisc"; }
		QString help() const { return "A simple impuls-height-window. The same values are used for all channels."; }

	private:
		NSEvaluation::Analyzer* _analyzer;
};

namespace NSEvaluation {
	class FilterExpression;
};

class MathFilter : public NSEvaluation::Filter
{
	public:
		MathFilter( NSCore::QTreeItem* );

		bool filter( NSCore::AcqisitionDataSet& );

		QString type() const { return "MathFilter"; }
		QString help() const { return "Filter on any arbitary math-expression."; }

	private:
		NSEvaluation::FilterExpression* _filter;
};

class Filters_Client : public NSEvaluation::PluginFactory_Client {
	public:
		NSEvaluation::Plugin* create( QString s, NSCore::QTreeItem* p ) {
			if ( s == "PeakFilter" )
				return new PeakFilter( p );
			if ( s == "TwoChannelDisc" )
				return new TwoChannelDisc( p );
			if ( s == "TwoChannelXORDisc" )
				return new TwoChannelXORDisc( p );
			if ( s == "SimpleDisc" )
				return new SimpleDisc( p );
			if ( s == "MathFilter" )
				return new MathFilter( p );
			return 0;
		}
		QStringList canCreate() {
			return QStringList()
				<< "PeakFilter"
				<< "TwoChannelDisc"
				<< "TwoChannelXORDisc"
				<< "SimpleDisc"
				<< "MathFilter"
			;
		}

		QString type( QString plugin ) {
			if ( canCreate().contains( plugin ) ) {
				return "Filter";
			}
			return "";
		}
};

#endif // FILTERS_H

