/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef SCRIPTINTROSPECTION_H
#define SCRIPTINTROSPECTION_H

#include <QtGui/QWidget>
#include <QtGui/QTreeWidgetItem>

namespace Ui {
	class ScriptIntrospection_Base;
};

namespace NSScript {

	class ScriptEnvironment;

	/**
	 * @brief A TreeView for introspection of scriptable objects.
	 *
	 * This displays a treeview with the methods and properties of objects added
	 * to a ScriptEnvironment.
	 *
	 * You have to call on_btnUpdate_clicked() to update ( and initialize ) the
	 * tree.
	 */
	class ScriptIntrospection : public QWidget
	{
		Q_OBJECT
		public:
			/// constructor
			/**
			 * @param env The ScriptEnvironment to introspect
			 * @param p The parent QWidget
			 */
			ScriptIntrospection( ScriptEnvironment* env, QWidget* p );

		public slots:
			/// Update the view
			void on_btnUpdate_clicked();

		private:
			/// Add a QObject (and its childs recursivly)
			void addQObject( QObject*, QTreeWidgetItem* );
			/// Add the properties of a QObject
			void addProperties( QObject*, QTreeWidgetItem* );
			/// Add the methods (signals/slots) of a QObject
			void addMethods( QObject*, QTreeWidgetItem* );

			ScriptEnvironment* _env;

			Ui::ScriptIntrospection_Base* _base;
	};

};

#endif // SCRIPTINTROSPECTION_H
