/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "scriptsnippet.h"
#include "scriptsnippet_private.h"
#include "scriptsnippet.moc"

#include <QtCore/QDebug>
#include <QtGui/QMessageBox>
#include <QtGui/QFileDialog>

using namespace NSScript;

ScriptSnippet::ScriptSnippet( QWidget* p )
	: QWidget( p )
	, _p( new ScriptSnippet_Private() )
	, _location( SystemWide )
{
	_p->setupUi( this );
}
ScriptSnippet::~ScriptSnippet() {
	emit destroyed( this );
}

QString ScriptSnippet::saveCode() const {
	return _p->edit->toPlainText();
}
void ScriptSnippet::setCode( QString code ) {
	_p->edit->setPlainText( code );
}
QString ScriptSnippet::name() const {
	return _p->nameEdit->text();
}
void ScriptSnippet::setName( QString t ) {
	_p->nameEdit->setText( t );
}

void ScriptSnippet::setSaveLocation( SaveLocation loc ) {
	_location = loc;
	_p->comboBoxLocation->setCurrentIndex( int( loc ) );
}

void ScriptSnippet::on_btnExec_clicked() {
	//qDebug() << "ScriptSnippet::on_btnExec_clicked()";
	QString state = _p->edit->toPlainText();
	if ( ( !_p->_states.isEmpty() && state != _p->_states.back() ) || _p->_states.isEmpty() ) {
		_p->_states.append( state );
		_p->_currentstate = _p->_states.size();
		_p->checkButtons();
	}
	emit execScript( state );
}

void ScriptSnippet::on_btnUndo_clicked() {
	qDebug() << "ScriptSnippet::on_btnUndo_clicked()";
}

void ScriptSnippet::on_btnRedo_clicked() {
	qDebug() << "ScriptSnippet::on_btnRedo_clicked()";
}

void ScriptSnippet::on_btnSave_clicked() {
	qDebug() << "ScriptSnippet::on_btnSave_clicked()";
	if ( _location == SystemWide ) {
		qDebug() << " SystemWide";
		emit systemSaveRequest();
		return;
	} // else:

	qDebug() << " OwnJsFile";

	QString filename = QFileDialog::getSaveFileName( this, "Save Script As", _p->nameEdit->text()+".js", "*.js\n*.*" );
	if ( ! filename.isEmpty() ) {
		QFile file( filename );
		if ( file.open( QIODevice::WriteOnly ) ) {
			file.write( _p->edit->toPlainText().toAscii() );
			file.close();
		}
	}
}

void ScriptSnippet::on_comboBoxLocation_activated( int item ) {
	if ( item == 2 )
		_p->btnSave->setEnabled( false );
	else
		_p->btnSave->setEnabled( true );

	_location = SaveLocation( item );
}

