
from math import *

def gauss( x, x0, s ):
	return 1/(sqrt(2*pi)*s) * exp( - pow(x-x0,2) / ( 2* pow(s,2) ) )

def decay( x, t ):
	if x >= 0:
		return exp( - x / t )
	return 0

def decayint( x, x0, s, t, A, ranged ):
	ret = 0
	i = ranged['start'] * 1.0
	ilist = [ i ]
	istep = ranged['step']
	while i < ranged['ende']:
		i += istep
		ilist.append(i)
	for i in ilist:
		tmp = gauss( i, x0, s ) * A * decay( x-i, t )
		#print "i = %i : %g" % ( i, tmp )
		ret += tmp
	#print " decayint(%g, %g, %g, %g, %g, %s) will return %g" % (x,x0,s,t,A,str(ranged),ret)
	return ret

class Data:
	"Structure to store the data"

	data = {}

	max = 0.
	maxat = 0.

	def __init__( self, filename ):
		f = file( filename, "r" )
		for line in f:
			line = line.strip("\r\n").split(" ")
			self.data[float(line[0])] = float(line[1])
		self.updateValues()

	def updateValues( self ):
		self.keys = self.data.keys()
		self.keys.sort()
		print "Old max: " + str((self.maxat,self.max))
		self.max = 0
		for i in self.keys:
			if self.data[i] > self.max:
				self.maxat = i
				self.max = self.data[i]
		print "New max: " + str((self.maxat,self.max))

	def __getitem__( self, d ):
		#print "Data::__getitem__( %s, %s )" % (str(self),str(d))
		return self.data[d]

	def __setitem__( self, d, n ):
		self.data[d] = n

	def determineNoise( self, range_to_use ):
		"Determine the noise by building a mean over the first and last range_to_use elements"
		noise = 0
		for i in range( range_to_use ) + range(len(self.keys)-5,len(self.keys)):
			noise = max( self.data[self.keys[i]], noise )
		#	noise += self.data[self.keys[i]]
		#noise /= range_to_use*2
		return noise

	def __isub__( self, sub ):
		" Subtract one value from all data-elements"
		for i in self.keys:
			self.data[i] -= sub
		self.updateValues()
		return self

class FitData(Data):

	def determineBoundaries( self ):
		keysrev = reversed( self.keys )
		begin = 0
		halfmax = 0
		for i in keysrev:
			if i < self.maxat and begin == 0:
				if self.data[i] < self.max/100:
					begin = i
			if i < self.maxat and halfmax == 0:
				if self.data[i] < self.max/2:
					halfmax = i
		end = 0
		for i in self.keys:
			if i > self.maxat and end == 0:
				if self.data[i] < self.max/10:
					end = i
		self.sigma = (self.maxat - halfmax) / 2
		self.begin = begin
		self.end = end
		print "FitData::determineBoundaries() new range will be [%g,%g]" % ( begin,end )

